@@ -4,9 +4,8 @@ use num_traits::{One, Zero};
 
 use tensorgraph_sys::{
     device::{DefaultDeviceAllocator, Device, DeviceAllocator},
-    ptr::reef::Ref,
-    vec::{DefaultVec, Vec},
-    Share, ShareMut,
+    ptr::Ref,
+    View, ViewMut, {DefaultVec, Vec},
 };
 
 use crate::{
@@ -22,13 +21,13 @@ pub struct Tensor<S: Storage, Dim: Dimension> {
     data: S,
 }
 
-impl<S: Storage, Dim: Dimension> Share for Tensor<S, Dim> {
+impl<S: Storage, Dim: Dimension> View for Tensor<S, Dim> {
     type Ref<'a>
     where
         Self: 'a,
-    = TensorView<'a, S::T, S::Device, Dim>;
+    = Tensor<&'a ViewOf<S>, Dim>;
 
-    fn share(&self) -> TensorView<S::T, S::Device, Dim> {
+    fn view(&self) -> TensorView<S::T, S::Device, Dim> {
         Tensor {
             shape: self.shape.clone(),
             strides: self.strides.clone(),
@@ -37,13 +36,13 @@ impl<S: Storage, Dim: Dimension> Share for Tensor<S, Dim> {
     }
 }
 
-impl<S: StorageMut, Dim: Dimension> ShareMut for Tensor<S, Dim> {
+impl<S: StorageMut, Dim: Dimension> ViewMut for Tensor<S, Dim> {
     type Mut<'a>
     where
         Self: 'a,
-    = TensorViewMut<'a, S::T, S::Device, Dim>;
+    = Tensor<&'a mut ViewOf<S>, Dim>;
 
-    fn share_mut(&mut self) -> TensorViewMut<S::T, S::Device, Dim> {
+    fn view_mut(&mut self) -> TensorViewMut<S::T, S::Device, Dim> {
         Tensor {
             shape: self.shape.clone(),
             strides: self.strides.clone(),
@@ -53,8 +52,10 @@ impl<S: StorageMut, Dim: Dimension> ShareMut for Tensor<S, Dim> {
 }
 
 impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
+    /// Creates a new tensor using the shape and the raw data.
+    /// The length of the data structure must match the size of the dimensions
     pub fn from_shape(shape: Dim, data: S) -> Self {
-        assert_eq!(data.as_ref().len(), shape.len());
+        assert_eq!(data.as_ref().len(), shape.size());
         let strides = shape.column_major_strides();
         Self {
             shape,
@@ -63,10 +64,12 @@ impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
         }
     }
 
+    /// Consumes the tensor, returning the underlying data
     pub fn into_inner(self) -> S {
         self.data
     }
 
+    /// Reverses the axes of the tensor. An inplace transpose
     pub fn reverse_axes(&mut self) {
         self.shape.as_mut().reverse();
         self.strides.as_mut().reverse();
@@ -77,12 +80,16 @@ impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
         self.strides.as_mut().swap(i, j);
     }
 
-    pub fn t(&self) -> TensorView<S::T, S::Device, Dim> {
-        let mut view = self.share();
+    /// Returns a view of the tensor with the contents transposed.
+    /// This operation happens without mutating or cloning any data
+    pub fn t(&self) -> Tensor<&ViewOf<S>, Dim> {
+        let mut view = self.view();
         view.reverse_axes();
         view
     }
 
+    /// Creates a new owned version of the tensor.
+    /// Will only clone the contents if needed
     pub fn into_owned(self) -> Tensor<S::Owned, Dim>
     where
         S: IntoOwned,
@@ -95,7 +102,8 @@ impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
         }
     }
 
-    pub fn slice_axis(&self, axis: usize, n: usize) -> TensorView<S::T, S::Device, Dim::Smaller>
+    /// Slices the tensor over a specific axis. The resulting tensor will be a dimension smaller
+    pub fn slice_axis(&self, axis: usize, n: usize) -> Tensor<&ViewOf<S>, Dim::Smaller>
     where
         Dim: RemoveDim,
     {
@@ -114,20 +122,36 @@ impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
     }
 }
 
-pub type TensorView<'a, T, D, Dim> = Tensor<&'a Ref<[T], D>, Dim>;
-pub type TensorViewMut<'a, T, D, Dim> = Tensor<&'a mut Ref<[T], D>, Dim>;
+/// A representation of a slice
+pub type Slice<T, D> = Ref<[T], D>;
+
+/// Gets the view repr of the provided storage
+pub type ViewOf<S> = Slice<<S as Storage>::T, <S as Storage>::Device>;
+
+/// A 'view' of a tensor, Like `&[T]` is to `Vec<T>`
+pub type TensorView<'a, T, D, Dim> = Tensor<&'a Slice<T, D>, Dim>;
+
+/// A 'mut view' of a tensor, Like `&mut [T]` is to `Vec<T>`
+pub type TensorViewMut<'a, T, D, Dim> = Tensor<&'a mut Slice<T, D>, Dim>;
+
+/// An uninit tensor. Contents are mutable and specified as [`MaybeUninit`].
 pub type UninitTensor<'a, T, D, Dim> = TensorViewMut<'a, MaybeUninit<T>, D, Dim>;
 
+/// A 2-dimensional tensor
 pub type Matrix<S> = Tensor<S, [usize; 2]>;
-pub type MatrixView<'a, T, D> = Matrix<&'a Ref<[T], D>>;
-pub type MatrixViewMut<'a, T, D> = Matrix<&'a mut Ref<[T], D>>;
+
+/// A 'view' of a matrix, Like `&[T]` is to `Vec<T>`
+pub type MatrixView<'a, T, D> = Matrix<&'a Slice<T, D>>;
+
+/// A 'mut view' of a matrix, Like `&mut [T]` is to `Vec<T>`
+pub type MatrixViewMut<'a, T, D> = Matrix<&'a mut Slice<T, D>>;
+
+/// An uninit matrix. Contents are mutable and specified as [`MaybeUninit`].
 pub type UninitMatrix<'a, T, D> = MatrixViewMut<'a, MaybeUninit<T>, D>;
 
 impl<S: Storage> Matrix<S> {
-    pub fn dot(
-        &self,
-        rhs: Matrix<impl Storage<T = S::T, Device = S::Device>>,
-    ) -> Matrix<DefaultVec<S::T, S::Device>>
+    /// Multiply two matricies together.
+    pub fn dot(&self, rhs: Matrix<&ViewOf<S>>) -> Matrix<DefaultVec<S::T, S::Device>>
     where
         S::T: Zero + One,
         S::Device: DefaultDeviceAllocator + DefaultBLASContext,
@@ -136,9 +160,10 @@ impl<S: Storage> Matrix<S> {
         self.dot_using(rhs, Default::default())
     }
 
+    /// Multiply two matricies together, using the specified [`BLASContext`]
     pub fn dot_using<C: BLASContext<S::Device>>(
         &self,
-        rhs: Matrix<impl Storage<T = S::T, Device = S::Device>>,
+        rhs: Matrix<&ViewOf<S>>,
         ctx: C,
     ) -> Matrix<DefaultVec<S::T, S::Device>>
     where
@@ -149,9 +174,10 @@ impl<S: Storage> Matrix<S> {
         self.dot_into(rhs, ctx, Default::default())
     }
 
+    /// Multiply two matricies together, using the provided [`DeviceAllocator`], using the specified [`BLASContext`]
     pub fn dot_into<C: BLASContext<S::Device>, A: DeviceAllocator<S::Device>>(
         &self,
-        rhs: Matrix<impl Storage<T = S::T, Device = S::Device>>,
+        rhs: Matrix<&ViewOf<S>>,
         ctx: C,
         alloc: A,
     ) -> Matrix<Vec<S::T, A, S::Device>>
@@ -166,7 +192,7 @@ impl<S: Storage> Matrix<S> {
             let uninit =
                 Matrix::from_shape([rows, cols], &mut v.space_capacity_mut()[..rows * cols]);
 
-            gemm_uninit_ctx(ctx, S::T::one(), self.share(), rhs, uninit);
+            gemm_uninit_ctx(ctx, S::T::one(), self.view(), rhs, uninit);
 
             v.set_len(rows * cols);
         }
@@ -174,7 +200,7 @@ impl<S: Storage> Matrix<S> {
     }
 }
 
-impl<'a, T: Copy, D: Device, Dim: Dimension> UninitTensor<'a, T, D, Dim> {
+impl<'a, T, D: Device, Dim: Dimension> UninitTensor<'a, T, D, Dim> {
     /// # Safety
     /// Contents must be initialised
     pub unsafe fn assume_init(self) -> TensorViewMut<'a, T, D, Dim> {
@@ -186,7 +212,7 @@ impl<'a, T: Copy, D: Device, Dim: Dimension> UninitTensor<'a, T, D, Dim> {
     }
 }
 
-impl<'a, T: Copy, D: Device, Dim: Dimension> TensorView<'a, MaybeUninit<T>, D, Dim> {
+impl<'a, T, D: Device, Dim: Dimension> TensorView<'a, MaybeUninit<T>, D, Dim> {
     /// # Safety
     /// Contents must be initialised
     pub unsafe fn assume_init(self) -> TensorView<'a, T, D, Dim> {
@@ -198,6 +224,10 @@ impl<'a, T: Copy, D: Device, Dim: Dimension> TensorView<'a, MaybeUninit<T>, D, D
     }
 }
 
+/// Performs the basic matmul operation.
+/// C = alpha * A * B.
+///
+/// Uses the default [`BLASContext`] for the device.
 pub fn gemm_uninit<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
     alpha: F,
     a: Matrix<impl Storage<T = F, Device = D>>,
@@ -207,6 +237,8 @@ pub fn gemm_uninit<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
     gemm_uninit_ctx(D::Context::default(), alpha, a, b, c)
 }
 
+/// Performs the basic matmul operation.
+/// C = alpha * A * B.
 pub fn gemm_uninit_ctx<F: GEMM<C, D> + Zero, C: BLASContext<D>, D: Device>(
     ctx: C,
     alpha: F,
@@ -219,6 +251,10 @@ pub fn gemm_uninit_ctx<F: GEMM<C, D> + Zero, C: BLASContext<D>, D: Device>(
     unsafe { gemm_ctx(ctx, alpha, a, b, F::zero(), c.assume_init()) }
 }
 
+/// Performs the basic matmul operation.
+/// C = alpha * A * B + beta * C.
+///
+/// Uses the default [`BLASContext`] for the device.
 pub fn gemm<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
     alpha: F,
     a: Matrix<impl Storage<T = F, Device = D>>,
@@ -229,6 +265,8 @@ pub fn gemm<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
     gemm_ctx(D::Context::default(), alpha, a, b, beta, c)
 }
 
+/// Performs the basic matmul operation.
+/// C = alpha * A * B + beta * C.
 pub fn gemm_ctx<F: GEMM<C, D> + Zero, C: BLASContext<D>, D: Device>(
     ctx: C,
     alpha: F,
@@ -289,7 +327,7 @@ fn lead(s: [usize; 2]) -> (MatrixOp, i32) {
 mod tests {
     use std::ops::Deref;
 
-    use tensorgraph_sys::{vec::Vec, Share, ShareMut};
+    use tensorgraph_sys::{Vec, View, ViewMut};
 
     use crate::tensor::{gemm, Tensor};
 
@@ -312,7 +350,7 @@ mod tests {
         // C = AB =  6 11
         //          10 19
 
-        let c = a.dot(b);
+        let c = a.dot(b.view());
         assert_eq!(c.into_inner().into_std(), [2., 6., 10., 3., 11., 19.]);
     }
 
@@ -344,7 +382,7 @@ mod tests {
         // C3 = A^B = 17 23
         //            39 53
 
-        let c3 = a.t().dot(b);
+        let c3 = a.t().dot(b.view());
         assert_eq!(c3.into_inner().into_std(), [17.0, 39.0, 23.0, 53.0]);
     }
 
@@ -402,7 +440,7 @@ mod tests {
         let a = Tensor::from_shape([3, 2], a);
         let b = Tensor::from_shape([2, 2], b);
 
-        let c = a.dot_into(b, ctx, cuda);
+        let c = a.dot_into(b.view(), ctx, cuda);
 
         let mut out = vec![0.0_f32; 6];
         c.data.copy_to_host(&mut out);
@@ -416,7 +454,7 @@ mod tests {
         use crate::blas::cublas::CublasContext;
         use tensorgraph_sys::{
             device::cuda::{Context, Cuda, Stream},
-            vec::DefaultVec,
+            DefaultVec,
         };
 
         let ctx = Context::quick_init().unwrap();
@@ -433,7 +471,7 @@ mod tests {
         let a = Tensor::from_shape([3, 2], a);
         let b = Tensor::from_shape([2, 2], b);
 
-        let c = a.dot(b);
+        let c = a.dot(b.view());
 
         let mut out = vec![0.0_f32; 6];
         c.data.copy_to_host(&mut out);
@@ -453,7 +491,7 @@ mod tests {
         let mut c = Tensor::from_shape([2, 2], c);
 
         for _ in 0..1000 {
-            gemm(1., a.share(), b.share(), 0., c.share_mut());
+            gemm(1., a.view(), b.view(), 0., c.view_mut());
             std::mem::swap(&mut a, &mut c);
         }
 
@@ -494,7 +532,7 @@ mod tests {
         let mut c = Tensor::from_shape([2, 2], c);
 
         for _ in 0..1000 {
-            gemm_ctx(ctx, 1., a.share(), b.share(), 0., c.share_mut());
+            gemm_ctx(ctx, 1., a.view(), b.view(), 0., c.view_mut());
             std::mem::swap(&mut a, &mut c);
         }
 
